#ifndef IMP_HPP_INCLUDED
#define IMP_HPP_INCLUDED


#include<cstdint>
#include<cstdio>


enum class Dir
{
  top,
  bottom,
  left,
  right,
  front,
  back,

  END,

};


struct
RenderingParameter
{
  int  side_length;

  int  width ;
  int  height;

  int  x_offset;
  int  y_offset;

  static constexpr int  calculate_width(int  side_length_, int  x_width, int  y_width)
  {
    return(side_length_*(x_width+y_width));
  }

  static constexpr int  calculate_height(int  side_length_, int  x_width, int  y_width, int  height)
  {
    return(calculate_y_offset(side_length_,height)+((side_length_/2)*(x_width+y_width+2)));
  }

  static constexpr int  calculate_x_offset(int  side_length_, int  x_width, int  y_width)
  {
    return((calculate_width(side_length_,x_width,y_width)/2)+((x_width < y_width)?  (side_length_/2)*((y_width-x_width)-2)
                                                                                 : -(side_length_/2)*((x_width-y_width)+2)));
  }

  static constexpr int  calculate_y_offset(int  side_length_, int  height)
  {
    return((side_length_/2)*((height*2)-2));
  }

  constexpr RenderingParameter(int  side_length_, int  x_width, int  y_width, int  height):
  side_length(side_length_),
   width(calculate_width(side_length_,x_width,y_width)),
  height(calculate_height(side_length_,x_width,y_width,height)),
  x_offset(calculate_x_offset(side_length_,x_width,y_width)),
  y_offset(calculate_y_offset(side_length_,height))
  {
  }

};




struct
Point3D
{
  int  x;
  int  y;
  int  z;

  constexpr Point3D(int  x_=0, int  y_=0, int  z_=0):
  x(x_), y(y_), z(z_)
  {
  }


  constexpr uint32_t  get_priority_number() const{return(x+y+z);}

  void  reset(int  x_=0, int  y_=0, int  z_=0)
  {
    x = x_;
    y = y_;
    z = z_;
  }

  Point3D  operator+(const Point3D&  rhs) const{return Point3D(x+rhs.x,y+rhs.y,z+rhs.z);}

  Point3D&  operator+=(const Point3D&  rhs)
  {
    x += rhs.x;
    y += rhs.y;
    z += rhs.z;

    return *this;
  }

  bool  operator==(const Point3D&  rhs) const
  {
    return((x == rhs.x) &&
           (y == rhs.y) &&
           (z == rhs.z));
  }

  void  print() const
  {
    printf("[print Point3D] X:%4d, Y:%4d, Z:%4d\n",x,y,z);
  }

};


struct
Point2D
{
  int  x;
  int  y;

  constexpr Point2D(int  x_=0, int  y_=0):
  x(x_), y(y_)
  {
  }

  constexpr Point2D(int  x_, int  y_, int  z_):
  x((x_  )-(y_  )   ),
  y((y_/2)+(x_/2)-z_)
  {
  }

  constexpr Point2D(const Point3D&  pt):
  x((pt.x  )-(pt.y  )     ),
  y((pt.y/2)+(pt.x/2)-pt.z)
  {
  }

};




struct
RenderingParameter2
{
  int  width ;
  int  height;

  int  x_offset;
  int  y_offset;

  constexpr RenderingParameter2(int  x_width, int  y_width, int  z):
   width(Point2D(x_width,      0,0).x*2),
  height(Point2D(x_width,y_width,0).y*2),
  x_offset(Point2D(x_width,0,0).x),
  y_offset(Point2D(x_width,y_width,0).y)
  {
  }

};




#endif




