#include"triplecube.hpp"
#include"cuberenderer.hpp"
#include"sincos.hpp"
#include<algorithm>




namespace{


TripleCube
tmp;


}


Point3D  TripleCube::base_view_point;
Point3D  TripleCube::   cursor_point;


void
TripleCube::
clear()
{
  std::memset(&elements_,0,sizeof(elements_));
}


void
TripleCube::
assign(const Cube&  cube)
{
  clear();

    rep(z,Cube::size){
    rep(y,Cube::size){
    rep(x,Cube::size){
      get(size+x,size+y,size+z) = cube.get(x,y,z);
    }}}


  source_cube = &cube;
}



      Cube::Element&  TripleCube::get(int  x, int  y, int  z)      {return elements_[(size*size*z)+(size*y)+x];}
const Cube::Element&  TripleCube::get(int  x, int  y, int  z) const{return elements_[(size*size*z)+(size*y)+x];}


void
TripleCube::
rotate_horizontally(int  center_x, int  center_y, int  angle)
{
  tmp.clear();

  x_min = size-1;
  x_max =      0;
  y_min = size-1;
  y_max =      0;
  z_min = size-1;
  z_max =      0;

  int  sin_value;
  int  cos_value;

  get_sincos(angle,sin_value,cos_value);

    rep(    z,size){
    rep(dst_y,size){
    rep(dst_x,size){
      int  src_x = (((dst_x-center_x) * cos_value - (dst_y-center_y) * sin_value) >> 10) + center_x;
      int  src_y = (((dst_x-center_x) * sin_value + (dst_y-center_y) * cos_value) >> 10) + center_y;

        if((src_x >=    0) &&
           (src_x <  size) &&
           (src_y >=    0) &&
           (src_y <  size))
        {
          auto&  e = get(src_x,src_y,z);

            if(e.mark != Cube::Mark::unuse)
            {
              tmp.get(dst_x,dst_y,z) = e;

              x_min = std::min(x_min,dst_x);
              x_max = std::max(x_max,dst_x);
              y_min = std::min(y_min,dst_y);
              y_max = std::max(y_max,dst_y);
              z_min = std::min(z_min,    z);
              z_max = std::max(z_max,    z);
            }
        }
    }}}


  std::memcpy(&elements_,&tmp.elements_,sizeof(elements_));
}


void
TripleCube::
rotate_vertically(int  center_x, int  center_z, int  angle)
{
  tmp.clear();

  x_min = size-1;
  x_max =      0;
  y_min = size-1;
  y_max =      0;
  z_min = size-1;
  z_max =      0;

  int  sin_value;
  int  cos_value;

  get_sincos(angle,sin_value,cos_value);

    rep(    y,size){
    rep(dst_z,size){
    rep(dst_x,size){
      int  src_x = (((dst_x-center_x) * cos_value - (dst_z-center_z) * sin_value) >> 10) + center_x;
      int  src_z = (((dst_z-center_z) * sin_value + (dst_z-center_z) * cos_value) >> 10) + center_z;

        if((src_x >=    0) &&
           (src_x <  size) &&
           (src_z >=    0) &&
           (src_z <  size))
        {
          auto&  e = get(src_x,y,src_z);

            if(e.mark != Cube::Mark::unuse)
            {
              tmp.get(dst_x,y,dst_z) = e;

              x_min = std::min(x_min,dst_x);
              x_max = std::max(x_max,dst_x);
              y_min = std::min(y_min,    y);
              y_max = std::max(y_max,    y);
              z_min = std::min(z_min,dst_z);
              z_max = std::max(z_max,dst_z);
            }
        }
    }}}


  std::memcpy(&elements_,&tmp.elements_,sizeof(elements_));
}


void
TripleCube::
update(Drawer&  dr, bool  xyz, bool  frame) const
{
    if(frame)
    {
      dr.draw_rect(0xFFFFFFFF,0,0,Cube::rendparam.width,Cube::rendparam.height);
    }


  CubeRenderer::clear();

  bool  found_flag = false;

    rep(z,size){
    rep(y,size){
    rep(x,size){
      auto&  elm = get(x,y,z);

        if(elm.mark != Cube::Mark::unuse)
        {
          auto&  focus = CubeRenderer::push(source_cube->palette_,elm,x,y,z);

            if(!found_flag &&
                ((cursor_point.x == x) &&
                 (cursor_point.y == y) &&
                 (cursor_point.z == z)))
            {
              found_flag = true;

              focus = 1;
            }
        }
    }}}



    if(!found_flag)
    {
      auto&  current = get(cursor_point.x,
                           cursor_point.y,
                           cursor_point.z);

      CubeRenderer::push(source_cube->palette_,current,cursor_point.x,
                                                       cursor_point.y,
                                                       cursor_point.z) = 1;
    }


  CubeRenderer::output(dr);

    if(xyz)
    {
      dr.draw_formatted(0xFFFFFFFF,0,          0,"X %2d",cursor_point.x);
      dr.draw_formatted(0xFFFFFFFF,0,font_size  ,"Y %2d",cursor_point.y);
      dr.draw_formatted(0xFFFFFFFF,0,font_size*2,"Z %2d",cursor_point.z);
    }
}





