#include"wirerenderer.hpp"
#include"sincos.hpp"
#include<algorithm>




const RenderingParameter2
WireRenderer::
rendparam(size,size,size);


const Point3D
WireRenderer::
center_point(size/2,size/2,size/2);


WireRenderer::
WireRenderer():
root_node(nullptr),
current_node(nullptr),
h_angle(0),
v_angle(0)
{
  cursor_node.offset_point.reset(size/2,size/2,size/3);
}




void
WireRenderer::
put(bool  filter)
{
    if(!current_node)
    {
      auto  new_node = new Node;

      new_node->offset_point = cursor_node.offset_point;
      new_node->pixel = 0xFFFFFFFF;

         root_node = new_node;
      current_node = new_node;

      cursor_node.offset_point.reset();
    }

  else
    if(!filter || !(current_node->calculated_point == cursor_node.calculated_point))
    {
      auto  new_node = new Node;

      new_node->offset_point = cursor_node.offset_point;
      new_node->pixel = 0xFFFFFFFF;

      current_node->append(*new_node);

      current_node = new_node;


      cursor_node.offset_point.reset();
    }
}


void
WireRenderer::
unput()
{
    if(current_node)
    {
      auto  p = current_node;
                current_node  = p->parent;

        if(current_node)
        {
          remove(current_node->children,p);
        }

      else
        {
          cursor_node.offset_point = p->offset_point;

          current_node = nullptr;
             root_node = nullptr;
        }


      delete p;
    }
}


bool
WireRenderer::
process(const Controller&  ctrl)
{
  constexpr int  v = 16;

    if(ctrl.test_pressing(Key::shift_flag))
    {
           if(ctrl.test_pressing(Key::up_flag  )){cursor_node.offset_point.z += v;}
      else if(ctrl.test_pressing(Key::down_flag)){cursor_node.offset_point.z -= v;}
    }

  else
    if(ctrl.test_pressing(Key::start_flag))
    {
      return true;
    }

  else
    {
           if(ctrl.test_pressing(Key::up_flag   )){cursor_node.offset_point.y -= v;}
      else if(ctrl.test_pressing(Key::left_flag )){cursor_node.offset_point.x -= v;}
      else if(ctrl.test_pressing(Key::right_flag)){cursor_node.offset_point.x += v;}
      else if(ctrl.test_pressing(Key::down_flag )){cursor_node.offset_point.y += v;}
      else if(ctrl.test_pressing(Key::p_flag)){  put();}
      else if(ctrl.test_pressing(Key::n_flag)){unput();}
    }


  return false;
}


namespace{
void
clamp(int&  v)
{
  v %= 360;

    if(v < 0)
    {
      v += 360;
    }
}
}


void
WireRenderer::
clamp_angles()
{
  clamp(h_angle);
  clamp(v_angle);
}


namespace{


void
draw_line(Drawer&  dr, const Point3D&  a_src, const Point3D&  b_src, int  h_angle, int  v_angle)
{
  Point3D    tmp;
  Point3D  a_dst;
  Point3D  b_dst;

  rotate_horizontally(a_src,WireRenderer::center_point,h_angle,  tmp);
    rotate_vertically(  tmp,WireRenderer::center_point,v_angle,a_dst);

  rotate_horizontally(b_src,WireRenderer::center_point,h_angle,  tmp);
    rotate_vertically(  tmp,WireRenderer::center_point,v_angle,b_dst);

  Node::draw_line(dr,to_pixel32(10,10,10),
                  a_dst.x,
                  a_dst.y,
                  a_dst.z,
                  b_dst.x,
                  b_dst.y,
                  b_dst.z);
}


}


void
WireRenderer::
update(Drawer&  dr)
{
  dr.draw_rect(0xFFFFFFFF,0,0,rendparam.width,rendparam.height);

  constexpr int  r = 32;

  constexpr Point3D  xa_point(size/2-r,size/2,size/2);
  constexpr Point3D  xb_point(size/2+r,size/2,size/2);

  constexpr Point3D  ya_point(size/2,size/2-r,size/2);
  constexpr Point3D  yb_point(size/2,size/2+r,size/2);

  constexpr Point3D  za_point(size/2,size/2,size/2-r);
  constexpr Point3D  zb_point(size/2,size/2,size/2+r);

  draw_line(dr,xa_point,xb_point,h_angle,v_angle);
  draw_line(dr,ya_point,yb_point,h_angle,v_angle);
  draw_line(dr,za_point,zb_point,h_angle,v_angle);

    if(root_node)
    {
      root_node->draw_as_root(dr,center_point,h_angle,v_angle);

      cursor_node.draw_as_stepchild(dr,*root_node,*current_node);
    }

  else
    {
      cursor_node.draw_as_root(dr,center_point,h_angle,v_angle);
    }


    {
      auto&  a_pt = cursor_node.offset_point;
      auto&  b_pt = cursor_node.calculated_point;

      dr.draw_formatted(0xFFFFFFFF,0,          0,"X %3d(%3d)",a_pt.x,b_pt.x);
      dr.draw_formatted(0xFFFFFFFF,0,font_size  ,"Y %3d(%3d)",a_pt.y,b_pt.y);
      dr.draw_formatted(0xFFFFFFFF,0,font_size*2,"Z %3d(%3d)",a_pt.z,b_pt.z);
      dr.draw_formatted(0xFFFFFFFF,0,font_size*3,"H ANGLE%3d",h_angle);
      dr.draw_formatted(0xFFFFFFFF,0,font_size*4,"V ANGLE%3d",v_angle);
    }
}




